/******************************************************************************
                               qgraphwidget.cpp
                             -------------------
  begin    : Thu Feb 21 2002
  copyright: (C) 2002 by Nikandrov George
  email    : gogan@ksi-linux.com
  release  : $Id: qgraphwidget.cpp,v 1.1.1.1 2002/04/18 17:13:20 gogan Exp $
 ******************************************************************************
 *                                                                            *
 *    This program is free software; you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
 *    the Free Software Foundation; either version 2 of the License, or       *
 *    (at your option) any later version.                                     *
 *                                                                            *
 *****************************************************************************/

#include "qgraphwidget.h"
#include "qplot.h"

QGraphWidget::QGraphWidget(QWidget *parent, const char *name) : QWidget(parent,name)
{
	p = 0;
	setBackgroundColor( white );
	mouse_button_down = false;
	p = new QPainter ();
	image = new QPixmap ( size() );
	image->fill();
	dad = NULL;
	accuracy = 1;
	xmin = -14.5;
	xmax =  14.5;
	ymin = -8.0;
	ymax =  8.0;
	values = new int[250];
	
	first_time = 2;
}

QGraphWidget::~QGraphWidget()
{
	delete p;
	delete image;
	delete[] values;
}

void QGraphWidget::setWidgetWithTimer(QPlot *plot)
{
        dad = plot;
}

void QGraphWidget::setColorsArray(QColor *coloravail)
{
	colors = coloravail;
}

void QGraphWidget::setScale(float Xmin, float Xmax, float Ymin, float Ymax)
{
    xmin = Xmin;
    xmax = Xmax;
    ymin = Ymin;
    ymax = Ymax;
    PlotAllFuncs();
}

void QGraphWidget::paintEvent( QPaintEvent * )
{
	bitBlt ( this, 0, 0, image );
}

void QGraphWidget::resizeEvent ( QResizeEvent * )
{
	image->resize(size());
	mywid = width();
	myhei = height();
	
	PlotAllFuncs();
}

void QGraphWidget::mousePressEvent ( QMouseEvent *e )
{
	mouse_button_down = true;
	xsave = e->x();
	ysave = e->y();
}

void QGraphWidget::mouseMoveEvent ( QMouseEvent *e )
{
	if ( mouse_button_down ) {
		xdelta = e->x() - xsave;
		ydelta = e->y() - ysave;
		if ( xdelta < 0 ) {
		    if ( ydelta < 0 ) {
			bitBlt ( this, 0, 0, image, -xdelta, -ydelta);
		    } else {
			bitBlt ( this, 0, ydelta, image, -xdelta, 0);
		    }
		} else {
		    if ( ydelta < 0 ) {
			bitBlt ( this, xdelta, 0, image, 0, -ydelta);
		    } else {
			bitBlt ( this, xdelta, ydelta, image, 0, 0);
		    }
		}
	}
}

void QGraphWidget::mouseReleaseEvent ( QMouseEvent *e )
{
	mouse_button_down = false;
	
	xdelta = e->x() - xsave;
	ydelta = e->y() - ysave;
	
	xscale = xdelta*(xmax-xmin)/width();
	yscale = ydelta*(ymax-ymin)/width();
	
	xmin -= xscale;
	xmax -= xscale;
	ymin += yscale;
	ymax += yscale;
			
	emit scaleChanged( xmin, xmax, ymin, ymax );
	PlotAllFuncs();
}

void QGraphWidget::drawCoords()
{
	int i;
		
	p->setPen(QColor(0,0,0));
	
	if ( (ymax >= 0) && (ymin <= 0) ) {
		p->drawLine(0,getYReal(0),mywid,getYReal(0));
		for ( i = (int)ceil ( xmin ) ; i <= (int)floor ( xmax ) ; i++ ) {
			p->drawLine(getXReal(i),getYReal(0)-MARK_WID,getXReal(i),getYReal(0)+MARK_WID);
		}
	} else {
		for ( i = (int)ceil ( xmin ) ; i <= (int)floor ( xmax ) ; i++ ) {
			p->drawLine(getXReal(i),0,getXReal(i),MARK_WID);
		}
	}
	
	if ( (xmax >= 0) && (xmin <= 0) ) {
		p->drawLine(getXReal(0),0,getXReal(0),myhei);
		for ( i = (int)ceil ( ymin ) ; i <= (int)floor ( ymax ) ; i++ ) {
		    p->drawLine(getXReal(0)-MARK_WID,getYReal(i),getXReal(0)+MARK_WID,getYReal(i));
		}
	} else {
		for ( i = (int)ceil ( ymin ) ; i <= (int)floor ( ymax ) ; i++ ) {
		    p->drawLine(0,getYReal(i),MARK_WID,getYReal(i));
		}
	}

}

void QGraphWidget::GraphPlot( unsigned int func_no, const QColor *fcol)
{
	char buffer[100];
	const char *answer;
	float cur_val;
	
	int i;
	int s;

	if ( dad != NULL ) {
		dad->abortTimer();
	}
	
	p->setPen(*fcol);
	
	saveScaleWorkaround(true);
	
	for (s=0; s<=mywid; s+=CHUNK_SIZE) {
		
		qApp->processEvents();

		for (i=s; i<s+CHUNK_SIZE; i++) {
		    saveScaleWorkaround(false);
		    sprintf(buffer,"f%d(%f)",func_no, xmin+i*(xmax-xmin)/(accuracy*mywid) );
		    sendCommand(buffer);
		}
		for (i=s; i<s+CHUNK_SIZE; i++) {
		    while ( ( answer=receiveResult() ) == NULL ) { usleep (100); };
		    if (sscanf(answer, "%f", &cur_val)!=0) {
			    values[i] = getYReal(cur_val);
		    } else {
			    values[i] = NAN;
		    }
		}
	}
	
	if ( values[0] != NAN ) {
	    p->moveTo(0,values[0]);
	}
	
	for (i=1; i<=mywid; i++) {
	    if ( (values[i] != NAN) && (values[i-1] != NAN) ) {
		if ( ( (values[i] - values[i-1]) > myhei ) ||
		     ( (values[i-1] - values[i]) > myhei )  ) {
			p->moveTo(i,values[i]);
		} else {
			p->lineTo(i,values[i]);
		}
	    } else {
		p->moveTo(i,values[i]);
	    }
	}
	
	p->flush();
	
	bitBlt ( this, 0, 0, image );

	saveScaleWorkaround(false);

	if ( dad != NULL ) {
		dad->resumeTimer();
	}
}

void QGraphWidget::PlotAllFuncs()
{
	qWarning("FirstTime: %d",first_time);
	if ( first_time > 0 ) {
	    first_time--;
	    return;
	}

	QCopEnvelope("QPE/System","busy()");
	qApp->processEvents();

	int z;
	
	image->fill();
	p->begin(image);
	drawCoords();
	bitBlt ( this, 0, 0, image );

	if ( dad != NULL ) {

	    if( (ymax-ymin) > 100 ) {
		z = 0;
	    } else if ( (ymax-ymin) > 0.2) {
		z = 3;
	    } else {
		z = 10;
	    }
	
	    sendSpecialVarDecCommand("scale",z);
	
	    for ( int i = 0; i< FUNCS_NUM; i++) {
		if (dad->funcs[i]->CheckBoxDraw->isChecked()) {
		    GraphPlot(i+1,&dad->colors[dad->funcs[i]->curColor]);
		}
	    }
	    
	    dad->setScale();
	}
		
	p->end();

	QCopEnvelope("QPE/System","notBusy()");
	QCopEnvelope e("QPE/System","notBusy(QString)");
	e << QString ("Done plotting");
	qApp->processEvents();
}


		