/******************************************************************************
                                  qplot.cpp
                             -------------------
  begin    : Sat Feb 2 2002
  copyright: (C) 2002 by Nikandrov George
  email    : gogan@ksi-linux.com
  release  : $Id: qplot.cpp,v 1.2 2002/04/19 09:13:11 gogan Exp $
 ******************************************************************************
 *                                                                            *
 *    This program is free software; you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
 *    the Free Software Foundation; either version 2 of the License, or       *
 *    (at your option) any later version.                                     *
 *                                                                            *
 *****************************************************************************/

#include "qplot.h"

QPlot::QPlot(QWidget *parent, const char *name ) : QPlotView(parent,name)
{
	saveIBaseDecimal();
	
	time_out = new QTimer ( this, "Timeout Timer" );
	lay = new QVBoxLayout( FuncsFrame ); 
	func_scroll = new QScrollView ( FuncsFrame, "Scroll" );
	func_lay = new QVBox( func_scroll->viewport() ); 
	lay->setSpacing( 0 );
	lay->setMargin( 0 );
	func_lay->setSpacing( 0 );
	func_lay->setMargin( 0 );
	lay->addWidget(func_scroll);
	func_scroll->addChild(func_lay);
	func_scroll->setHScrollBarMode(QScrollView::AlwaysOff);

	funcs = new QFuncEdit *[FUNCS_NUM];
		
	colors = new QColor[MAX_COLORS];
	colors[0] = QColor(0,0,0);
	colors[1] = QColor(127,0,0);
	colors[2] = QColor(0,127,0);
	colors[3] = QColor(0,0,127);
	colors[4] = QColor(127,127,0);
	colors[5] = QColor(0,127,127);
	colors[6] = QColor(127,0,127);

#ifdef QWS
	Config cfg("QPlot");
	cfg.setGroup("Misc");
	
	if ( cfg.readNumEntry("FirstTime",1) ) {
	DialogField->append("(At the bottom, you can see some tabs.\n"
				"If you scroll to the right using buttons\n"
				"on the right bottom, you will see 'Help' tab)\n\n");
	}
	cfg.writeEntry("FirstTime", 0);

	cfg.setGroup("Functions");
#endif

	for (int i=0; i<FUNCS_NUM; i++) {
	    funcs[i] = new QFuncEdit( func_lay , i+1,  colors, "FuncEdit");
#ifdef QWS
	    funcs[i]->LineEditFunc->setText( cfg.readEntry( funcs[i]->LabelNumber->text() ) );
	    funcs[i]->ButtonFT->setText( cfg.readEntry( funcs[i]->LabelNumber->text() + QString("t"), "y" ) );
	    funcs[i]->curColor = cfg.readNumEntry( funcs[i]->LabelNumber->text() + QString("c"), 0) - 1;
	    funcs[i]->setColor();
	    funcs[i]->CheckBoxDraw->setChecked( (bool)cfg.readNumEntry( funcs[i]->LabelNumber->text() + QString("d"), 0) );
	    QPEApplication::setStylusOperation( funcs[i]->LineEditFunc, QPEApplication::RightOnHold );	    
#endif
	    sendFunction(i+1, funcs[i]->LineEditFunc->text().latin1() );
	}
	
	
	GraphView->setWidgetWithTimer(this);
	GraphView->setColorsArray(colors);

	DialogField->setCursorPosition(DialogField->numLines()+1,1);
	DialogField->setFocus();

	connect(time_out, SIGNAL( timeout() ), this, SLOT( acceptResult() ) );
	time_out->start(500);
	
#ifdef QWS
	QPEApplication::setStylusOperation( this, QPEApplication::RightOnHold );
	QPEApplication::setStylusOperation( DialogField, QPEApplication::RightOnHold );
#endif

	fileSelector = new FileSelector( "text/x-bc", tab_7, "fileselector" , TRUE, FALSE );
        tabLayout_7->addWidget( fileSelector );

//	connect( fileSelector, SIGNAL( closeMe() ), this, SLOT( showEditTools() ) );
	connect( fileSelector, SIGNAL( newSelected( const DocLnk &) ), this, SLOT( NewProgrammingFile( const DocLnk & ) ) );
	connect( fileSelector, SIGNAL( fileSelected( const DocLnk &) ), this, SLOT( OpenProgrammingFile( const DocLnk & ) ) );


	buffer = NULL;
	
	setGraphPreset(0);

	Global::statusMessage(tr ("Please use QPlot inputmethod."));	
}

//-------------------------------------------------------------------------

QPlot::~QPlot()
{
	delete[] funcs;
}

//-------------------------------------------------------------------------

void QPlot::abortTimer()
{
	time_out->stop();
}

//-------------------------------------------------------------------------

void QPlot::resumeTimer()
{
	time_out->start(500);
}

//-------------------------------------------------------------------------

void QPlot::executeStatement()
{
	sendCommand( DialogField->textLine(DialogField->numLines()-2) );
	usleep(500);
	acceptResult();
}

//-------------------------------------------------------------------------
void QPlot::acceptResult()
{
	while ( ( buffer=receiveResult() ) != NULL ) {
		DialogField->append(buffer);
		DialogField->setCursorPosition ( DialogField->numLines()+1, 1 );
		usleep(50);
	}
}
//-------------------------------------------------------------------------

void QPlot::clearScreen()
{
	abortTimer();
	resetEngineState();
	resumeTimer();
	DialogField->clear();
}

//-------------------------------------------------------------------------

void QPlot::callQuit()
{
	qApp->quit();
}

//-------------------------------------------------------------------------

void QPlot::setScale()
{
	sendSpecialVarDecCommand("scale",SpinBoxScale->value());
	sendSpecialVarDecCommand("obase",SpinBoxOBase->value());
	sendSpecialVarDecCommand("ibase",SpinBoxIBase->value(),false);
	sendCommand("pi=2*very_long_pi/2");
	sendCommand("e=2*very_long_e/2");
}

//-------------------------------------------------------------------------

void QPlot::FeedFunctions()
{
#ifdef QWS
	Config cfg("QPlot");
	cfg.setGroup("Functions");
#endif
	
	for ( int i = 0; i< FUNCS_NUM; i++) {		
		sendFunction(i+1,funcs[i]->LineEditFunc->text().latin1());
#ifdef QWS
		cfg.writeEntry( funcs[i]->LabelNumber->text(),
				    funcs[i]->LineEditFunc->text() );
		cfg.writeEntry( funcs[i]->LabelNumber->text() + QString("c"),
				    funcs[i]->curColor);
		cfg.writeEntry( funcs[i]->LabelNumber->text() + QString("d"),
				    (int)funcs[i]->CheckBoxDraw->isChecked() );
		cfg.writeEntry( funcs[i]->LabelNumber->text() + QString("t"),
				    funcs[i]->ButtonFT->text() );
#endif
	}
}

//-------------------------------------------------------------------------

void QPlot::updateXYminmax(float xmin, float xmax, float ymin, float ymax)
{
	QString s;
	s.setNum(xmin);
	LineEditXmin->setText(s);
	s.setNum(xmax);
	LineEditXmax->setText(s);
	s.setNum(ymin);
	LineEditYmin->setText(s);
	s.setNum(ymax);
	LineEditYmax->setText(s);
}

//------------------------------------------------------------------------

void QPlot::setGraphScale()
{
	
	GraphView->setScale(LineEditXmin->text().toFloat(),
				LineEditXmax->text().toFloat(),
				LineEditYmin->text().toFloat(),
				LineEditYmax->text().toFloat());
}

//------------------------------------------------------------------------

void QPlot::setGraphPreset(int num)
{
	switch (num) {
	    case 0:	updateXYminmax(-6.3,6.3,-3.1,3.1);	//Default
			setGraphScale();
			break;
	    case 1:	updateXYminmax(-9.4247779,9.4247779,-4,4);	//Trig
			setGraphScale();
			break;
	    case 2:	updateXYminmax(-14.5,14.5,-8,8);	//Big
			setGraphScale();
			break;
	}
}

//------------------------------------------------------------------------

void QPlot::setAngMode(int num)
{
	sendSpecialVarDecCommand("angle_mode",num);
}

//------------------------------------------------------------------------

void QPlot::SolveNewton()
{
	time_out->stop();
	
	char *buffer;
	
	AnswerLabel->setText(tr("Please wait, solving..."));
	
	buffer = new char[1000];

	sendCommand("define eq_right(x) {");
	sprintf(buffer, "return (%s);", ComboBoxRightPart->currentText().latin1() );
	sendCommand(buffer);
	sendCommand("}");

	sendCommand("define eq_left(x) {");
	sprintf(buffer, "return (%s);", ComboBoxLeftPart->currentText().latin1() );
	sendCommand(buffer);
	sendCommand("}");

	sendCommand("define equation(x) {\nreturn (eq_left(x) - eq_right(x));\n}");
	
	sprintf(buffer, "equation_solve(%s);", LineEditStartVal->text().latin1() );
	sendCommand("verbose=1;");
	sendCommand(buffer);
	sendCommand("verbose=0;");
	
	delete buffer;
	
	QString s;
	
	while ( ( buffer=receiveResult() ) == NULL ) {usleep (100);}
	s += buffer;
	while ( ( buffer=receiveResult() ) == NULL ) {usleep (100);}
	s += buffer;
	while ( ( buffer=receiveResult() ) == NULL ) {usleep (100);}
	s += buffer;
	while ( ( buffer=receiveResult() ) != NULL ) {
	    usleep (100);
	    s += buffer;
	}
	
	s.replace(QRegExp("\\"),"\\\n");
	
	AnswerLabel->setText(s);
		
	time_out->start(500);
}

//------------------------------------------------------------------------

void QPlot::SolveQuadratic()
{
	time_out->stop();
	
	char *buffer;
	
	QuadAnswerLabel->setText( tr ("Please wait, solving..." ) );
	
	buffer = new char[1000];

	sprintf(buffer, "equation_solve_quad(%s,%s,%s);", LineEditQCA->text().latin1(),
							    LineEditQCB->text().latin1(),
							    LineEditQCC->text().latin1() );
	sendCommand(buffer);
	
	delete buffer;
	
	QString s;
	
	while ( ( buffer=receiveResult() ) == NULL ) {usleep (100);}
	s += buffer;
	while ( ( buffer=receiveResult() ) == NULL ) {usleep (100);}
	s += buffer;
	while ( ( buffer=receiveResult() ) != NULL ) {
	    usleep (100);
	    s += buffer;
	}
	
	s.replace(QRegExp("\\"),"\\\n");

	QuadAnswerLabel->setText(s);
		
	time_out->start(500);
}

//--------------------------------------------------------------------------

void QPlot::ZoomIn2D()
{
	float xdelta, ydelta;	

	QString s;
	
	xdelta = (LineEditXmax->text().toFloat()-LineEditXmin->text().toFloat())*0.1;
	ydelta = (LineEditYmax->text().toFloat()-LineEditYmin->text().toFloat())*0.1;
	
	s.setNum(LineEditXmin->text().toFloat() + xdelta);
	LineEditXmin->setText(s);
	s.setNum(LineEditYmin->text().toFloat() + ydelta);
	LineEditYmin->setText(s);
	
	s.setNum(LineEditXmax->text().toFloat() - xdelta);
	LineEditXmax->setText(s);
	s.setNum(LineEditYmax->text().toFloat() - ydelta);
	LineEditYmax->setText(s);
	
	setGraphScale();
}

//--------------------------------------------------------------------------

void QPlot::ZoomOut2D()
{
	float xdelta, ydelta;	

	QString s;
	
	xdelta = (LineEditXmax->text().toFloat()-LineEditXmin->text().toFloat())*0.1;
	ydelta = (LineEditYmax->text().toFloat()-LineEditYmin->text().toFloat())*0.1;
	
	s.setNum(LineEditXmin->text().toFloat() - xdelta);
	LineEditXmin->setText(s);
	s.setNum(LineEditYmin->text().toFloat() - ydelta);
	LineEditYmin->setText(s);
	
	s.setNum(LineEditXmax->text().toFloat() + xdelta);
	LineEditXmax->setText(s);
	s.setNum(LineEditYmax->text().toFloat() + ydelta);
	LineEditYmax->setText(s);
	
	setGraphScale();
}

//--------------------------------------------------------------------------
void QPlot::setDocument(const QString &fn)
{
	DocLnk doc(fn);
	
	OpenProgrammingFile(doc);
}


//--------------------------------------------------------------------------

void QPlot::OpenProgrammingFile(const DocLnk &f)
{
	clearScreen();

	FileManager fm;
	QIODevice *iod = fm.openFile(f);
	int c;
	while ( ( c = iod->getch() ) >= 0 ) {
	    putc( c, stderr );
	    putchar ( c );
	}
	
	putchar('\n');
	putchar('\n');

	delete iod;
	
	MainMenuTab->setCurrentPage(0);
}

//--------------------------------------------------------------------------

void QPlot::NewProgrammingFile(const DocLnk &f)
{
	bool ok = FALSE;
	
	QString filename = QInputDialog::getText( tr("New Program"),
			    tr("Please type in desired program name.\n"
			       "Text Editor will be started to edit it."),
			    QString::null, &ok, this );
			    
	if (ok && !filename.isEmpty() ) {			    
		DocLnk nf = f;
		nf.setType("text/x-bc");
		nf.setName(filename);
		nf.writeLink();

	        FileManager fm;
		if ( fm.saveFile( nf, "/* QPlot Program */\n" ) ) {
		        QCopEnvelope e("QPE/Application/textedit","setDocument(QString)");
			e << nf.linkFile();
	        }
	}
}

