/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <qapplication.h>
#include <qpixmap.h>

#include "qplotmath.h"
#include "qplotmathimpl.h"

#include "math.xpm"

QPlotMathImpl::QPlotMathImpl()
    : input(0), icn(0), ref(0)
{
}

QPlotMathImpl::~QPlotMathImpl()
{
    delete input;
    delete icn;
}

QWidget *QPlotMathImpl::inputMethod( QWidget *parent, Qt::WFlags f )
{
    if ( !input )
	input = new QPlotMath( parent, "QPlotMath", f);
    return input;
}

void QPlotMathImpl::resetState()
{
    if ( input )
	input->resetState();
}

QPixmap *QPlotMathImpl::icon()
{
    if ( !icn )
	icn = new QPixmap( (const char **)math_xpm );
    return icn;
}

QString QPlotMathImpl::name()
{
    return qApp->translate( "InputMethods", "QPlot-Style Math" );
}

void QPlotMathImpl::onKeyPress( QObject *receiver, const char *slot )
{
    if ( input )
	QObject::connect( input, SIGNAL(key(ushort,ushort,ushort,bool,bool)), receiver, slot );
}

#ifndef QT_NO_COMPONENT
QRESULT QPlotMathImpl::queryInterface( const QUuid &uuid, QUnknownInterface **iface )
{
    *iface = 0;
    if ( uuid == IID_QUnknown )
	*iface = this;
    else if ( uuid == IID_InputMethod )
	*iface = this;

    if ( *iface )
	(*iface)->addRef();
    return QS_OK;
}

Q_EXPORT_INTERFACE()
{
    Q_CREATE_INSTANCE( QPlotMathImpl )
}
#endif
